function simuData= DGP_irregular(simuPar)
%%
% simuData = DGP_irregular(simuPar)
% 
% Simulate noisy log-prices. The efficient price process has
% stochastic volatilities and exhibits jumps in both price and volatility.
% The stationary part of noise follows an AR(1) model, see Ait-Sahalia, Mykland and Zhang(2011).
% The size process of noise follows and OU-process and the observation
% schemes follows an inhomogeneous Poisson process.
% The number of intraday observations have a U-pattern thus Irregular scheme.  
% 
% Inputs:        simuPar --- parameters of the efficient price and noise processes
%                       see the specifications below
% Outputs:     simuData.y --- simulated observed noisy prices
%                     simuData.X --- simulated efficient prices
%                     simuData.QV --- Quardratic variation of the efficient
%                     price
%                     simuData.noise --- simulated noise
%                     simuData.G --- simulated gamma process, the noise
%                     size process
%                     simuData.obvts --- simulated observation times
% 
% References: 
% 
% (1) Ait-Sahalia, Yacine, Per A. Mykland, and Lan Zhang. 
% "Ultra high frequency volatility estimation with dependent microstructure noise."
% Journal of Econometrics 160.1 (2011): 160-175.
% 
%(2) Jacod, J., Y. Li, and  X. Zheng (2017): ?Statistical properties of microstructure noise,? 
% Econometrica, 85, 1133?1174.
%
% (3) Li, Z. Merrick and Oliver Linton (2020):
% "A ReMeDI for microstructure noise."
% 
% Authors : Z. Merrick Li and Oliver Linton
% 
% 
%%
%% Random observation times
n = simuPar.NumObv;
dt0= exprnd(1/n,2*n,1);
dt = zeros(2*n,1);
dt(1) = dt0(1)/((1+cos(0*2*pi)/2));
for i = 2:(2*n)
    dt(i)  =  dt0(i)/...
        (1+cos(sum(dt(1:i-1))*2*pi)/2);
end
obvts  =  cumsum(dt);
dt = dt(obvts<=1);% time lags between obvs
obvts = obvts(obvts<=1);
obvts = [0;obvts];% random observation times
n = length(obvts);
%% Efficient price and noise size processes
% efficient price related parameters
kappa1 = simuPar.kappa1;
kappa2 = simuPar.kappa2;
mu1 = simuPar.mu1;
mu2 = simuPar.mu2;
gamma0 = simuPar.gamma;
varrho = simuPar.varrho;
rho = simuPar.rho;
lambda = simuPar.lambda;
rhoG = simuPar.rhoG;
gc = simuPar.gc;
%muG = simuPar.muG;
sigmaG = simuPar.sigmaG;
% simulate the two BMs
sigmaW = [1,varrho;varrho,1];
dWs = mvnrnd([0,0],sigmaW,n-1);
% simulate the two jump processes
jumptimes = binornd(ones(size(dt)),dt*lambda);
sigma2jumpsize1 = mu2/10;
if simuPar.pricejump == 1
     jumpsizes1 = sqrt(sigma2jumpsize1)*randn(size(dt));
 else
    jumpsizes1 = zeros(size(dt));
end

if simuPar.voljump == 1
    jumpsizes2 = exprnd(gamma0,size(dt));
else
    jumpsizes2 = zeros(size(dt));
end
J1 = jumptimes.*jumpsizes1;
J2 = jumptimes.*jumpsizes2;
% add drift, diffusion and jumps
Xs = zeros(n,1);
Gs = zeros(n,1);
sigma2s = zeros(n,1);
Xs(1) = mu1;
Gs(1) = (1+0.1*cos(2*pi*0));
sigma2s(1) = mu2;
for i = 2:n
    dsigma2 = kappa2*(mu2-sigma2s(i-1))*dt(i-1)...
      + gamma0*sqrt(sigma2s(i-1)*dt(i-1))*dWs(i-1,2)...
      + J2(i-1);
    sigma2s(i) = sigma2s(i-1) + dsigma2;
    dX = kappa1*(mu1-Xs(i-1))*dt(i-1) + ...
        sqrt(sigma2s(i-1)*dt(i-1))*dWs(i-1,1) ...
        + J1(i-1);
    Xs(i) = Xs(i-1) + dX;
    dG = -rhoG*(Gs(i-1) - (1+0.1*cos(2*pi*(i-1)/n)))...
        *dt(i-1) + sqrt(sigmaG^2*dt(i-1))*dWs(i-1,1);
    % noise size process Gamma
    Gs(i) = Gs(i-1) + dG;
end
IV = sum(sigma2s(2:n).*dt);
QV = IV + sum(J1.^2);
%% simulate the noise process
% simulate the stationary ar(1) process
ar1model = arima('Constant',0,'AR',{rho},'Variance',1-rho^2);
epsilons = simulate (ar1model, n);
% get the size parameters
Gs = gc*Gs;
noise = Gs.*epsilons;
%% simulate the observed prices
y = Xs + noise;
simuData.y = y;
simuData.X = Xs;
simuData.QV = QV;
simuData.noise = noise;
simuData.G = Gs;
simuData.obvts = obvts;
simuData.dt = dt;
%%
end