function [y, X, noise]= DGP_regular(simuPar)
%%
% [y, X, noise]= DGP_regular(simuPar)
% 
% Simulate noisy log-prices. The efficient price process has
% stochastic volatilities and exhibits jumps in both price and volatility.
% The noise follows an AR(1) model, see Ait-Sahalia, Mykland and Zhang(2011).
% 
% Inputs: simuPar --- parameters of the efficient price and noise processes
%                     see the specifications below
% Outputs: y --- the observed noisy price, which is sum of Xs and noise
%          X --- the (latent) efficient price and related parameters
%          noise --- the (latent) microstructure noise
% 
% References: 
% 
% (1) Ait-Sahalia, Yacine, Per A. Mykland, and Lan Zhang. 
% "Ultra high frequency volatility estimation with dependent microstructure noise."
% Journal of Econometrics 160.1 (2011): 160-175.
% 
% (2) Li, Z. Merrick and Oliver Linton (2020):
% "A ReMeDI for microstructure noise."
% 
% Authors : Z. Merrick Li and Oliver Linton
% 
% 
%%
%% Determine the irregular observation times
delta = simuPar.delta;
n = floor(simuPar.T/delta);
obvtimes = delta*(0:n);
n = length(obvtimes);
%dobvtimes = obvtimes(2:n)-obvtimes(1:n-1);
%dobvtimes = [obvtimes(1);dobvtimes];
%% extract the parameters
% efficient price related parameters
kappa1 = simuPar.kappa1;
kappa2 = simuPar.kappa2;
mu1 = simuPar.mu1;
mu2 = simuPar.mu2;
gamma0 = simuPar.gamma;
varrho = simuPar.varrho;
rho = simuPar.rho;
lambda = simuPar.lambda;
% noise size parameters
gc = simuPar.gc;
%% simulate the efficient price process
% simulate the two BMs
sigmaW = [1,varrho;varrho,1];
dWs = mvnrnd([0,0],sigmaW,n-1);
% simulate the two jump processes
jumptimes = binornd(ones(n,1),delta*lambda);
sigma2jumpsize1 = mu2/10;
if simuPar.pricejump == 1
     jumpsizes1 = sqrt(sigma2jumpsize1)*randn(n,1);
 else
    jumpsizes1 = zeros(n,1);
end

if simuPar.voljump == 1
    jumpsizes2 = exprnd(gamma0,n,1);
else
    jumpsizes2 = zeros(n,1);
end
J1 = jumptimes.*jumpsizes1;
J2 = jumptimes.*jumpsizes2;
% add drift, diffusion and jumps
Xs = zeros(n,1);
sigma2s = zeros(n,1);
Xs(1) = mu1;
sigma2s(1) = mu2;
for i = 2:n
    dsigma2 = kappa2*(mu2-sigma2s(i-1))*delta...
              + gamma0*sqrt(sigma2s(i-1)*delta)*dWs(i-1,2)...
              + J2(i);
    sigma2s(i) = sigma2s(i-1) + dsigma2;
    dX = kappa1*(mu1-Xs(i-1))*delta + ...
        sqrt(sigma2s(i-1)*delta)*dWs(i-1,1) ...
        + J1(i-1);
    Xs(i) = Xs(i-1) + dX;
end
IV = sum(sigma2s*delta);
QV = IV + sum(J1.^2);
X.logp = Xs;
X.QV = QV;
%% simulate the noise process
% simulate the stationary ar(1) process
ar1model = arima('Constant',0,'AR',{rho},'Variance',1-rho^2);
epsilons = simulate (ar1model, n);
% get the size parameters
noise = gc*epsilons;
%% simulate the observed prices
y = Xs + noise;
%%
end