function Noise = LA2(logprice, kn, jstar, QVest)
%%
% Noise = LA2(logprice, kn,jstar,QVest)
%
% Estimate the variance and covariance of noise by the
% local averaging estimator. 
%
% Inputs: logprice --- log price
%         kn --- the size of local averaging
%         QVest --- an estimate of the integrated volatility, can be
%         obtained via the multi-scale RV estimators, it will be used in
%         the bias correction step
%
% Outputs: Noise --- a struct containing all the relavent estimates of
%          second order moments of noise
%
% References: 
%
% Jacod, J., Y. Li, and X. Zheng (2017): 
% "Statistical properties of microstructure noise,"
% Econometrica, 85, 1133  --- 1174.
%
%
% Authors : Z. Merrick Li and Oliver Linton
%
%%
y = logprice;
n = length(y);
%% perform the estimates
varNoise = JLZCovEst(y,0,kn);
% correct the small sample bias
if nargin == 4
    varNoiseADJ = JLZCovEst(y,0,kn) - kn*QVest*4/3/n;
    covNoiseADJ = [];
end
covNoise = [];
for j = 1:jstar-1
    covj = JLZCovEst(y,j,kn);
    covNoise = [covNoise; covj];
    % correct the small sample bias
    if nargin == 4
        covjADJ = covj - kn*QVest*4/3/n;
        covNoiseADJ = [covNoiseADJ; covjADJ];
    end
end
Noise.cov = [varNoise;covNoise];
if nargin == 4
    Noise.cov_adj = [varNoiseADJ;covNoiseADJ];
end
end

function covj = JLZCovEst(Y,j,kn)
%%
% Estimate the (co)variance of noise based on the estimators
% proposed in Jacod et al. (2017)
% Inputs: 
%               Y ---  observed log-price
%               j --- the order of covariance, e.g., j = 0 corresponds to
%               the variance of noise.
%               kn --- the size of local average
%
% Output: covj --- the j-th covariance 
%
% References: 
%
% Jacod, J., Y. Li, and X. Zheng (2017): 
% "Statistical properties of microstructure noise,"
% Econometrica, 85, 1133  1174.
%
%
% Authors : Z. Merrick Li and Oliver Linton
%
% Date : 01 July 2018
%%
n = length(Y);

%% compute the local averages
% no local average for the first j+kn obvs
YLA = zeros(j+kn,1);
for i = 1+j+kn:n-kn+1
    ylatemp = mean(Y(i:i+kn-1));
    YLA = [YLA;ylatemp];
end
% no local average for the last kn-1 obvs
YLA = [YLA;zeros(kn-1,1)];
covj = 0;
%% compute the JLZ estimators the j-th covariance 
for i = 1:n+1-j-4*kn
    covj = covj + (Y(i) - YLA(i+j+kn))*(Y(i+j) - YLA(i+j+3*kn));
end
covj = covj/n;
end
    

    

